@@ -23,6 +23,7 @@ import           System.IO.Unsafe (unsafePerformIO)
 import           System.Process
 import           Text.PrettyPrint.ANSI.Leijen hiding ((<$>))
 import           System.Directory
+import           System.Directory.Internal.Prelude
 
 -- | Handle for holding temporary resources, the @postgres@ process handle
 --   and @postgres@ connection information. The 'DB' also includes the
@@ -131,6 +132,10 @@ fastPostgresConfig =
   , ("log_min_error_statement", "PANIC")
   , ("log_statement", "none")
   , ("client_min_messages", "ERROR")
+  , ("commit_delay", "100000")
+  , ("wal_level", "minimal")
+  , ("archive_mode", "off")
+  , ("max_wal_senders", "0")
   ]
 
 {-|
@@ -150,6 +155,10 @@ log_min_messages = PANIC
 log_min_error_statement = PANIC
 log_statement = none
 client_min_messages = ERROR
+commit_delay = 100000
+wal_level = minimal
+archive_mode = off
+max_wal_senders = 0
 @
 
 'defaultConfig' also passes the @--no-sync@ flag to @initdb@.
@@ -180,10 +189,6 @@ custom = defaultConfig <> mempty
   }
 @
 
-
- This is common enough there is `defaultPostgresConf` which
- is a helper to do this.
-
  As an alternative to using 'defaultConfig' one could create a
  config from connections parameters using 'optionsToDefaultConfig'.
 
@@ -219,24 +224,6 @@ defaultConfig_9_3_10 = mempty
     }
   }
 
-
-{-|
-'mappend' the 'defaultConfig' with a 'Config' that provides additional
-   \"postgresql.conf\" lines. Equivalent to:
-
-@
-'defaultPostgresConf' extra = 'defaultConfig' <> mempty
-  { 'postgresConfigFile' = extra
-  }
-@
-
-@since 1.21.0.0
--}
-defaultPostgresConf :: [(String, String)] -> Config
-defaultPostgresConf extra = defaultConfig <> mempty
-  { postgresConfigFile = extra
-  }
-
 -- | Default postgres options
 --
 --   @since 1.21.0.0
@@ -247,12 +234,19 @@ verbosePostgresConfig =
   , ("synchronous_commit", "off")
   , ("full_page_writes", "off")
   , ("log_min_duration_statement", "0")
-  , ("log_connections", "on")
-  , ("log_disconnections", "on")
   , ("client_min_messages", "WARNING")
   , ("log_min_messages", "WARNING")
   , ("log_min_error_statement", "WARNING")
-  , ("log_statement", "all")
+  , ("log_checkpoints", "on")
+  , ("log_connections", "on")
+  , ("log_disconnections", "on")
+  , ("log_lock_waits", "on")
+  , ("log_temp_files", "0")
+  , ("log_autovacuum_min_duration", "0")
+  , ("log_error_verbosity", "default")
+  , ("log_line_prefix", "'%t [%p]: '")
+  , ("lc_messages", "'C'")
+  , ("track_io_timing", "on")
   ]
 
 {-|
@@ -631,6 +625,9 @@ withDbCache $ \\cache -> withConfig (cacheConfig cache) $ \\db ->
     withConfig (snapshotConfig db) $ \\migratedDb -> ...
 @
 
+The 'Snapshot's are ephemeral. If you would like the 'Snapshot's to persistent
+consider using 'cacheAction' instead.
+
 @since 1.29.0.0
 -}
 withSnapshot
@@ -665,6 +662,7 @@ snapshotConfig = fromFilePathConfig . toFilePath . unSnapshot
 -------------------------------------------------------------------------------
 -- cacheAction
 -------------------------------------------------------------------------------
+
 {-|
 Check to see if a cached data directory exists.
 
@@ -679,7 +677,11 @@ on startup onto the @initial@ config and returns it. In other words:
 initialConfig <> configFromCachePath
 @
 
-@since 1.29.0.0
+'cacheAction' can be used to create a snapshot of migrated database and not
+remigrate as long as the migration does not change. See 'withSnapshot' for
+a ephemeral version of taking snapshots.
+
+@since 1.32.0.0
 -}
 cacheAction
   :: FilePath
@@ -692,18 +694,20 @@ cacheAction
 cacheAction cachePath action config = do
   fixCachePath <- fixPath cachePath
   let result = config <> fromFilePathConfig fixCachePath
-  nonEmpty <- doesFileExist $ fixCachePath <> "/PG_VERSION"
 
-  case nonEmpty of
-    True -> pure $ pure result
-    False -> fmap join $ withConfig config $ \db -> do
-      action db
-      -- TODO see if parallel is better
-      throwIfNotSuccess id =<< stopPostgresGracefully db
-      createDirectoryIfMissing True fixCachePath
+  -- TODO document the change in behavior
+  preexisting <- try (createDirectory fixCachePath) >>= \case
+    Left e | isAlreadyExistsError e -> pure True
+    Left e -> throwIO e
+    Right () -> pure False
 
-      let snapshotCopyCmd = cpFlags <>
-            toDataDirectory db <> "/* " <> fixCachePath
-      system snapshotCopyCmd >>= \case
-        ExitSuccess -> pure $ pure result
-        x -> pure $ Left $ SnapshotCopyFailed snapshotCopyCmd x
+  if preexisting then pure $ pure result else fmap join $ withConfig config $ \db -> do
+    -- TODO see if parallel is better
+    action db `onException` removeDirectoryRecursive fixCachePath
+    throwIfNotSuccess id =<< stopPostgresGracefully db
+
+    let snapshotCopyCmd = cpFlags <>
+          toDataDirectory db <> "/* " <> fixCachePath
+    system snapshotCopyCmd >>= \case
+      ExitSuccess -> pure $ pure result
+      x -> pure $ Left $ SnapshotCopyFailed snapshotCopyCmd x
