@@ -353,10 +353,18 @@ instance Semigroup DirectoryType where
 instance Monoid DirectoryType where
   mempty = Temporary
 
--- | Either create a'CTemporary' directory or do nothing to a 'CPermanent'
+fixPath :: FilePath -> IO FilePath
+fixPath x = case x of
+  '~':rest -> do
+    homeDir <- getHomeDirectory
+    pure $ homeDir <> "/" <> rest
+  xs -> pure xs
+
+-- | Either create a'CTemporary' directory or do create the directory
+--   if it does not exist to a 'CPermanent'
 --   one.
 --
---   @since 1.12.0.0
+--   @since 1.29.0.0
 setupDirectoryType
   :: String
   -- ^ Temporary directory configuration
@@ -364,13 +372,9 @@ setupDirectoryType
   -- ^ Directory pattern
   -> DirectoryType
   -> IO CompleteDirectoryType
-setupDirectoryType tempDir pat = \case
+setupDirectoryType tempDir pat dirType = case dirType of
   Temporary -> CTemporary <$> createTempDirectory tempDir pat
-  Permanent x  -> CPermanent <$> case x of
-    '~':rest -> do
-      homeDir <- getHomeDirectory
-      pure $ homeDir <> "/" <> rest
-    xs -> pure xs
+  Permanent x  -> CPermanent <$> fixPath x
 
 -- Remove a temporary directory and ignore errors
 -- about it not being there.
@@ -385,9 +389,9 @@ rmDirIgnoreErrors mainDir = do
   -- TODO come up with a better way to deal with this. Probably
   -- need to lock the directories recursively before deleting.
   handle ignoreDirIsMissing $
-    try (removePathForcibly mainDir) >>= \case
-    Left (_ :: IOError) -> try (removePathForcibly mainDir) >>= \case
-      Left (_ :: IOError) -> removePathForcibly mainDir
+    try (removeDirectoryRecursive mainDir) >>= \case
+    Left (_ :: IOError) -> try (removeDirectoryRecursive mainDir) >>= \case
+      Left (_ :: IOError) -> removeDirectoryRecursive mainDir
       Right _ -> pure ()
     Right _ -> pure ()
 
@@ -596,7 +600,8 @@ getInitDbVersion = unsafePerformIO $ readProcessWithExitCode "initdb" ["--versio
 
 makeCommandLine :: String -> CompleteProcessConfig -> String
 makeCommandLine command CompleteProcessConfig {..} =
-  let envs = unwords $ map (\(x, y) -> x <> "=" <> y) completeProcessConfigEnvVars
+  let envs = unwords $ map (\(x, y) -> x <> "=" <> y)
+             $ filter ((`elem` envsToKeep) . fst) completeProcessConfigEnvVars
       args = unwords completeProcessConfigCmdLine
   in envs <> " " <> command <> args
 
@@ -613,6 +618,39 @@ makeCachePath cacheFolder cmdLine =
     theHash = makeArgumentHash cmdLine
   in cacheFolder <> "/" <> version <> "/" <> theHash
 
+envsToKeep :: [String]
+envsToKeep =
+  [ "PGHOST"
+  , "PGHOSTADDR"
+  , "PGPORT"
+  , "PGDATABASE"
+  , "PGUSER"
+  , "PGPASSWORD"
+  , "PGPASSFILE"
+  , "PGSERVICE"
+  , "PGSERVICEFILE"
+  , "PGOPTIONS"
+  , "PGAPPNAME"
+  , "PGSSLMODE"
+  , "PGREQUIRESSL"
+  , "PGSSLCOMPRESSION"
+  , "PGSSLCERT"
+  , "PGSSLKEY"
+  , "PGSSLROOTCERT"
+  , "PGSSLCRL"
+  , "PGREQUIREPEER"
+  , "PGKRBSRVNAME"
+  , "PGGSSLIB"
+  , "PGCONNECT_TIMEOUT"
+  , "PGCLIENTENCODING"
+  , "PGTARGETSESSIONATTRS"
+  , "PGDATESTYLE"
+  , "PGTZ"
+  , "PGGEQO"
+  , "PGSYSCONFDIR"
+  , "PGLOCALEDIR"
+  ]
+
 splitDataDirectory :: CompleteProcessConfig -> (Maybe String, CompleteProcessConfig)
 splitDataDirectory old =
   let isDataDirectoryFlag xs = "-D" `isPrefixOf` xs || "--pgdata=" `isPrefixOf` xs
@@ -878,4 +916,3 @@ hostToSocketClass :: String -> DirectoryType
 hostToSocketClass hostOrSocketPath = case hostOrSocketPath of
   '/' : _ -> Permanent hostOrSocketPath
   _ -> Temporary
-
