@@ -1,4 +1,5 @@
 import           Control.Concurrent
+import qualified Control.Concurrent.Async as Async
 import           Control.Exception
 import           Control.Monad ((<=<), void, unless)
 import           Data.Function
@@ -27,6 +28,7 @@ import           System.Posix.Files
 import           System.Process
 import           System.Timeout
 import           Test.Hspec
+import           Data.Either
 
 withConn :: DB -> (PG.Connection -> IO a) -> IO a
 withConn db f = do
@@ -379,12 +381,11 @@ happyPaths = describe "succeeds with" $ do
 
   it "withDbCache seems to work" $
     withDbCache $ \cacheInfo ->
-      either throwIO pure =<< withConfig (cacheConfig cacheInfo) assertConnection
+      either throwIO pure <=< withConfig (cacheConfig cacheInfo <> verboseConfig) $ \db -> do
+        assertConnection db
+        withConn db $ \conn -> countDbs conn `shouldReturn` 3
+
 
-  it "postgresql.conf append last wins" $
-    withConfig' (verboseConfig <> defaultPostgresConf [("fsync", "on")]) $ \db -> do
-      toPostgresqlConfigFile db `shouldContain` "fsync=on"
-      withConn db $ \conn -> countDbs conn `shouldReturn` 3
 --
 -- Error Plans. Can't be combined. Just list them out inline since they can't be combined
 --
@@ -629,6 +630,34 @@ cacheActionSpecs = describe "cacheAction" $ do
         Left (SnapshotCopyFailed {}) -> pure ()
         _ -> fail $ "cacheAction should have failed with SnapshotCopyFailed"
 
+  it "works if two threads try to create a cache at the same time" $ do
+    withTempDirectory "/tmp" "tmp-postgres-parallel-cache-test" $ \dirPath -> do
+      withDbCache $ \cacheInfo -> do
+        lock <- newEmptyMVar
+        let
+          theConfig = defaultConfig <> cacheConfig cacheInfo
+          waitIfSecond _ = do
+            tryPutMVar lock () >>= \case
+              True -> pure ()
+              False -> threadDelay 100000
+          theCacheAction = cacheAction dirPath waitIfSecond theConfig
+        res <- Async.replicateConcurrently 3 theCacheAction
+        if all isRight res then pure () else fail "Failed to create caches concurrently"
+
+  it "nested calls don't deadlock" $ do
+    withTempDirectory "/tmp" "tmp-postgres-cache-action" $ \cachePath -> do
+      let
+        action _ = do
+          let differentCachePath = cachePath <> "/cached1"
+          cacheAction differentCachePath (const $ pure ()) defaultConfig >>= \case
+            Left err -> fail $ "Second call should succeed but failed with " <> show err
+            Right _ -> pure ()
+        theFinalCachePath = cachePath <> "/cached"
+
+      cacheAction theFinalCachePath action defaultConfig >>= \case
+        Left err -> fail $ "First call should succeed but failed with " <> show err
+        Right _ -> pure ()
+
 spec :: Spec
 spec = do
   withConfigSpecs
@@ -681,11 +710,12 @@ spec = do
     shouldSatisfy (Set.fromList $ words dbString) $
       Set.isSubsetOf wordsToSearchFor
 
-  let justBackupResources = defaultPostgresConf
-        [ ("wal_level", "replica")
-        , ("archive_mode","on")
-        , ("max_wal_senders","2")
-        ]
+  let justBackupResources = mempty { postgresConfigFile =
+          [ ("wal_level", "replica")
+          , ("archive_mode","on")
+          , ("max_wal_senders","2")
+          ]
+        }
       backupResources = justBackupResources
 
   it "can support backup and restore" $ withConfig' backupResources $ \db@DB {..} -> do
